/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/s3/S3_EXPORTS.h>
#include <aws/s3/model/RequestCharged.h>

#include <utility>

namespace Aws {
template <typename RESULT_TYPE>
class AmazonWebServiceResult;

namespace Utils {
namespace Xml {
class XmlDocument;
}  // namespace Xml
}  // namespace Utils
namespace S3 {
namespace Model {
class RestoreObjectResult {
 public:
  AWS_S3_API RestoreObjectResult() = default;
  AWS_S3_API RestoreObjectResult(const Aws::AmazonWebServiceResult<Aws::Utils::Xml::XmlDocument>& result);
  AWS_S3_API RestoreObjectResult& operator=(const Aws::AmazonWebServiceResult<Aws::Utils::Xml::XmlDocument>& result);

  ///@{

  inline RequestCharged GetRequestCharged() const { return m_requestCharged; }
  inline void SetRequestCharged(RequestCharged value) {
    m_requestChargedHasBeenSet = true;
    m_requestCharged = value;
  }
  inline RestoreObjectResult& WithRequestCharged(RequestCharged value) {
    SetRequestCharged(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Indicates the path in the provided S3 output location where Select results
   * will be restored to.</p>
   */
  inline const Aws::String& GetRestoreOutputPath() const { return m_restoreOutputPath; }
  template <typename RestoreOutputPathT = Aws::String>
  void SetRestoreOutputPath(RestoreOutputPathT&& value) {
    m_restoreOutputPathHasBeenSet = true;
    m_restoreOutputPath = std::forward<RestoreOutputPathT>(value);
  }
  template <typename RestoreOutputPathT = Aws::String>
  RestoreObjectResult& WithRestoreOutputPath(RestoreOutputPathT&& value) {
    SetRestoreOutputPath(std::forward<RestoreOutputPathT>(value));
    return *this;
  }
  ///@}

  ///@{

  inline const Aws::String& GetRequestId() const { return m_requestId; }
  template <typename RequestIdT = Aws::String>
  void SetRequestId(RequestIdT&& value) {
    m_requestIdHasBeenSet = true;
    m_requestId = std::forward<RequestIdT>(value);
  }
  template <typename RequestIdT = Aws::String>
  RestoreObjectResult& WithRequestId(RequestIdT&& value) {
    SetRequestId(std::forward<RequestIdT>(value));
    return *this;
  }
  ///@}
 private:
  RequestCharged m_requestCharged{RequestCharged::NOT_SET};

  Aws::String m_restoreOutputPath;

  Aws::String m_requestId;
  bool m_requestChargedHasBeenSet = false;
  bool m_restoreOutputPathHasBeenSet = false;
  bool m_requestIdHasBeenSet = false;
};

}  // namespace Model
}  // namespace S3
}  // namespace Aws
