/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/core/utils/memory/stl/AWSVector.h>
#include <aws/kms/KMS_EXPORTS.h>
#include <aws/kms/model/RotationsListEntry.h>

#include <utility>

namespace Aws {
template <typename RESULT_TYPE>
class AmazonWebServiceResult;

namespace Utils {
namespace Json {
class JsonValue;
}  // namespace Json
}  // namespace Utils
namespace KMS {
namespace Model {
class ListKeyRotationsResult {
 public:
  AWS_KMS_API ListKeyRotationsResult() = default;
  AWS_KMS_API ListKeyRotationsResult(const Aws::AmazonWebServiceResult<Aws::Utils::Json::JsonValue>& result);
  AWS_KMS_API ListKeyRotationsResult& operator=(const Aws::AmazonWebServiceResult<Aws::Utils::Json::JsonValue>& result);

  ///@{
  /**
   * <p>A list of completed key material rotations. When the optional input parameter
   * <code>IncludeKeyMaterial</code> is specified with a value of
   * <code>ALL_KEY_MATERIAL</code>, this list includes the first key material and any
   * imported key material pending rotation.</p>
   */
  inline const Aws::Vector<RotationsListEntry>& GetRotations() const { return m_rotations; }
  template <typename RotationsT = Aws::Vector<RotationsListEntry>>
  void SetRotations(RotationsT&& value) {
    m_rotationsHasBeenSet = true;
    m_rotations = std::forward<RotationsT>(value);
  }
  template <typename RotationsT = Aws::Vector<RotationsListEntry>>
  ListKeyRotationsResult& WithRotations(RotationsT&& value) {
    SetRotations(std::forward<RotationsT>(value));
    return *this;
  }
  template <typename RotationsT = RotationsListEntry>
  ListKeyRotationsResult& AddRotations(RotationsT&& value) {
    m_rotationsHasBeenSet = true;
    m_rotations.emplace_back(std::forward<RotationsT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>When <code>Truncated</code> is true, this element is present and contains the
   * value to use for the <code>Marker</code> parameter in a subsequent request.</p>
   */
  inline const Aws::String& GetNextMarker() const { return m_nextMarker; }
  template <typename NextMarkerT = Aws::String>
  void SetNextMarker(NextMarkerT&& value) {
    m_nextMarkerHasBeenSet = true;
    m_nextMarker = std::forward<NextMarkerT>(value);
  }
  template <typename NextMarkerT = Aws::String>
  ListKeyRotationsResult& WithNextMarker(NextMarkerT&& value) {
    SetNextMarker(std::forward<NextMarkerT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>A flag that indicates whether there are more items in the list. When this
   * value is true, the list in this response is truncated. To get more items, pass
   * the value of the <code>NextMarker</code> element in this response to the
   * <code>Marker</code> parameter in a subsequent request.</p>
   */
  inline bool GetTruncated() const { return m_truncated; }
  inline void SetTruncated(bool value) {
    m_truncatedHasBeenSet = true;
    m_truncated = value;
  }
  inline ListKeyRotationsResult& WithTruncated(bool value) {
    SetTruncated(value);
    return *this;
  }
  ///@}

  ///@{

  inline const Aws::String& GetRequestId() const { return m_requestId; }
  template <typename RequestIdT = Aws::String>
  void SetRequestId(RequestIdT&& value) {
    m_requestIdHasBeenSet = true;
    m_requestId = std::forward<RequestIdT>(value);
  }
  template <typename RequestIdT = Aws::String>
  ListKeyRotationsResult& WithRequestId(RequestIdT&& value) {
    SetRequestId(std::forward<RequestIdT>(value));
    return *this;
  }
  ///@}
 private:
  Aws::Vector<RotationsListEntry> m_rotations;

  Aws::String m_nextMarker;

  bool m_truncated{false};

  Aws::String m_requestId;
  bool m_rotationsHasBeenSet = false;
  bool m_nextMarkerHasBeenSet = false;
  bool m_truncatedHasBeenSet = false;
  bool m_requestIdHasBeenSet = false;
};

}  // namespace Model
}  // namespace KMS
}  // namespace Aws
