/**
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * SPDX-License-Identifier: Apache-2.0.
 */

#pragma once
#include <aws/core/utils/DateTime.h>
#include <aws/core/utils/memory/stl/AWSMap.h>
#include <aws/core/utils/memory/stl/AWSString.h>
#include <aws/s3/S3Request.h>
#include <aws/s3/S3_EXPORTS.h>
#include <aws/s3/model/ChecksumAlgorithm.h>
#include <aws/s3/model/ChecksumType.h>
#include <aws/s3/model/ObjectCannedACL.h>
#include <aws/s3/model/ObjectLockLegalHoldStatus.h>
#include <aws/s3/model/ObjectLockMode.h>
#include <aws/s3/model/RequestPayer.h>
#include <aws/s3/model/ServerSideEncryption.h>
#include <aws/s3/model/StorageClass.h>

#include <utility>

namespace Aws {
namespace Http {
class URI;
}  // namespace Http
namespace S3 {
namespace Model {

/**
 */
class CreateMultipartUploadRequest : public S3Request {
 public:
  AWS_S3_API CreateMultipartUploadRequest() = default;

  // Service request name is the Operation name which will send this request out,
  // each operation should has unique request name, so that we can get operation's name from this request.
  // Note: this is not true for response, multiple operations may have the same response name,
  // so we can not get operation's name from response.
  inline virtual const char* GetServiceRequestName() const override { return "CreateMultipartUpload"; }

  AWS_S3_API Aws::String SerializePayload() const override;

  AWS_S3_API void AddQueryStringParameters(Aws::Http::URI& uri) const override;

  AWS_S3_API Aws::Http::HeaderValueCollection GetRequestSpecificHeaders() const override;

  AWS_S3_API bool HasEmbeddedError(IOStream& body, const Http::HeaderValueCollection& header) const override;
  /**
   * Helper function to collect parameters (configurable and static hardcoded) required for endpoint computation.
   */
  AWS_S3_API EndpointParameters GetEndpointContextParams() const override;

  ///@{
  /**
   * <p>The canned ACL to apply to the object. Amazon S3 supports a set of predefined
   * ACLs, known as <i>canned ACLs</i>. Each canned ACL has a predefined set of
   * grantees and permissions. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#CannedACL">Canned
   * ACL</a> in the <i>Amazon S3 User Guide</i>.</p> <p>By default, all objects are
   * private. Only the owner has full access control. When uploading an object, you
   * can grant access permissions to individual Amazon Web Services accounts or to
   * predefined groups defined by Amazon S3. These permissions are then added to the
   * access control list (ACL) on the new object. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/S3_ACLs_UsingACLs.html">Using
   * ACLs</a>. One way to grant the permissions using the request headers is to
   * specify a canned ACL with the <code>x-amz-acl</code> request header.</p>
   * <ul> <li> <p>This functionality is not supported for directory buckets.</p>
   * </li> <li> <p>This functionality is not supported for Amazon S3 on Outposts.</p>
   * </li> </ul>
   */
  inline ObjectCannedACL GetACL() const { return m_aCL; }
  inline bool ACLHasBeenSet() const { return m_aCLHasBeenSet; }
  inline void SetACL(ObjectCannedACL value) {
    m_aCLHasBeenSet = true;
    m_aCL = value;
  }
  inline CreateMultipartUploadRequest& WithACL(ObjectCannedACL value) {
    SetACL(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The name of the bucket where the multipart upload is initiated and where the
   * object is uploaded.</p> <p> <b>Directory buckets</b> - When you use this
   * operation with a directory bucket, you must use virtual-hosted-style requests in
   * the format <code>
   * <i>Bucket-name</i>.s3express-<i>zone-id</i>.<i>region-code</i>.amazonaws.com</code>.
   * Path-style requests are not supported. Directory bucket names must be unique in
   * the chosen Zone (Availability Zone or Local Zone). Bucket names must follow the
   * format <code> <i>bucket-base-name</i>--<i>zone-id</i>--x-s3</code> (for example,
   * <code> <i>amzn-s3-demo-bucket</i>--<i>usw2-az1</i>--x-s3</code>). For
   * information about bucket naming restrictions, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/directory-bucket-naming-rules.html">Directory
   * bucket naming rules</a> in the <i>Amazon S3 User Guide</i>.</p> <p> <b>Access
   * points</b> - When you use this action with an access point for general purpose
   * buckets, you must provide the alias of the access point in place of the bucket
   * name or specify the access point ARN. When you use this action with an access
   * point for directory buckets, you must provide the access point name in place of
   * the bucket name. When using the access point ARN, you must direct requests to
   * the access point hostname. The access point hostname takes the form
   * <i>AccessPointName</i>-<i>AccountId</i>.s3-accesspoint.<i>Region</i>.amazonaws.com.
   * When using this action with an access point through the Amazon Web Services
   * SDKs, you provide the access point ARN in place of the bucket name. For more
   * information about access point ARNs, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/using-access-points.html">Using
   * access points</a> in the <i>Amazon S3 User Guide</i>.</p>  <p>Object
   * Lambda access points are not supported by directory buckets.</p>  <p>
   * <b>S3 on Outposts</b> - When you use this action with S3 on Outposts, you must
   * direct requests to the S3 on Outposts hostname. The S3 on Outposts hostname
   * takes the form <code>
   * <i>AccessPointName</i>-<i>AccountId</i>.<i>outpostID</i>.s3-outposts.<i>Region</i>.amazonaws.com</code>.
   * When you use this action with S3 on Outposts, the destination bucket must be the
   * Outposts access point ARN or the access point alias. For more information about
   * S3 on Outposts, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/S3onOutposts.html">What
   * is S3 on Outposts?</a> in the <i>Amazon S3 User Guide</i>.</p>
   */
  inline const Aws::String& GetBucket() const { return m_bucket; }
  inline bool BucketHasBeenSet() const { return m_bucketHasBeenSet; }
  template <typename BucketT = Aws::String>
  void SetBucket(BucketT&& value) {
    m_bucketHasBeenSet = true;
    m_bucket = std::forward<BucketT>(value);
  }
  template <typename BucketT = Aws::String>
  CreateMultipartUploadRequest& WithBucket(BucketT&& value) {
    SetBucket(std::forward<BucketT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies caching behavior along the request/reply chain.</p>
   */
  inline const Aws::String& GetCacheControl() const { return m_cacheControl; }
  inline bool CacheControlHasBeenSet() const { return m_cacheControlHasBeenSet; }
  template <typename CacheControlT = Aws::String>
  void SetCacheControl(CacheControlT&& value) {
    m_cacheControlHasBeenSet = true;
    m_cacheControl = std::forward<CacheControlT>(value);
  }
  template <typename CacheControlT = Aws::String>
  CreateMultipartUploadRequest& WithCacheControl(CacheControlT&& value) {
    SetCacheControl(std::forward<CacheControlT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies presentational information for the object.</p>
   */
  inline const Aws::String& GetContentDisposition() const { return m_contentDisposition; }
  inline bool ContentDispositionHasBeenSet() const { return m_contentDispositionHasBeenSet; }
  template <typename ContentDispositionT = Aws::String>
  void SetContentDisposition(ContentDispositionT&& value) {
    m_contentDispositionHasBeenSet = true;
    m_contentDisposition = std::forward<ContentDispositionT>(value);
  }
  template <typename ContentDispositionT = Aws::String>
  CreateMultipartUploadRequest& WithContentDisposition(ContentDispositionT&& value) {
    SetContentDisposition(std::forward<ContentDispositionT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies what content encodings have been applied to the object and thus
   * what decoding mechanisms must be applied to obtain the media-type referenced by
   * the Content-Type header field.</p>  <p>For directory buckets, only the
   * <code>aws-chunked</code> value is supported in this header field.</p>
   */
  inline const Aws::String& GetContentEncoding() const { return m_contentEncoding; }
  inline bool ContentEncodingHasBeenSet() const { return m_contentEncodingHasBeenSet; }
  template <typename ContentEncodingT = Aws::String>
  void SetContentEncoding(ContentEncodingT&& value) {
    m_contentEncodingHasBeenSet = true;
    m_contentEncoding = std::forward<ContentEncodingT>(value);
  }
  template <typename ContentEncodingT = Aws::String>
  CreateMultipartUploadRequest& WithContentEncoding(ContentEncodingT&& value) {
    SetContentEncoding(std::forward<ContentEncodingT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The language that the content is in.</p>
   */
  inline const Aws::String& GetContentLanguage() const { return m_contentLanguage; }
  inline bool ContentLanguageHasBeenSet() const { return m_contentLanguageHasBeenSet; }
  template <typename ContentLanguageT = Aws::String>
  void SetContentLanguage(ContentLanguageT&& value) {
    m_contentLanguageHasBeenSet = true;
    m_contentLanguage = std::forward<ContentLanguageT>(value);
  }
  template <typename ContentLanguageT = Aws::String>
  CreateMultipartUploadRequest& WithContentLanguage(ContentLanguageT&& value) {
    SetContentLanguage(std::forward<ContentLanguageT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>A standard MIME type describing the format of the object data.</p>
   */
  inline const Aws::String& GetContentType() const { return m_contentType; }
  inline bool ContentTypeHasBeenSet() const { return m_contentTypeHasBeenSet; }
  template <typename ContentTypeT = Aws::String>
  void SetContentType(ContentTypeT&& value) {
    m_contentTypeHasBeenSet = true;
    m_contentType = std::forward<ContentTypeT>(value);
  }
  template <typename ContentTypeT = Aws::String>
  CreateMultipartUploadRequest& WithContentType(ContentTypeT&& value) {
    SetContentType(std::forward<ContentTypeT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The date and time at which the object is no longer cacheable.</p>
   */
  inline const Aws::Utils::DateTime& GetExpires() const { return m_expires; }
  inline bool ExpiresHasBeenSet() const { return m_expiresHasBeenSet; }
  template <typename ExpiresT = Aws::Utils::DateTime>
  void SetExpires(ExpiresT&& value) {
    m_expiresHasBeenSet = true;
    m_expires = std::forward<ExpiresT>(value);
  }
  template <typename ExpiresT = Aws::Utils::DateTime>
  CreateMultipartUploadRequest& WithExpires(ExpiresT&& value) {
    SetExpires(std::forward<ExpiresT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specify access permissions explicitly to give the grantee READ, READ_ACP, and
   * WRITE_ACP permissions on the object.</p> <p>By default, all objects are private.
   * Only the owner has full access control. When uploading an object, you can use
   * this header to explicitly grant access permissions to specific Amazon Web
   * Services accounts or groups. This header maps to specific permissions that
   * Amazon S3 supports in an ACL. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html">Access
   * Control List (ACL) Overview</a> in the <i>Amazon S3 User Guide</i>.</p> <p>You
   * specify each grantee as a type=value pair, where the type is one of the
   * following:</p> <ul> <li> <p> <code>id</code> – if the value specified is the
   * canonical user ID of an Amazon Web Services account</p> </li> <li> <p>
   * <code>uri</code> – if you are granting permissions to a predefined group</p>
   * </li> <li> <p> <code>emailAddress</code> – if the value specified is the email
   * address of an Amazon Web Services account</p>  <p>Using email addresses to
   * specify a grantee is only supported in the following Amazon Web Services
   * Regions: </p> <ul> <li> <p>US East (N. Virginia)</p> </li> <li> <p>US West (N.
   * California)</p> </li> <li> <p> US West (Oregon)</p> </li> <li> <p> Asia Pacific
   * (Singapore)</p> </li> <li> <p>Asia Pacific (Sydney)</p> </li> <li> <p>Asia
   * Pacific (Tokyo)</p> </li> <li> <p>Europe (Ireland)</p> </li> <li> <p>South
   * America (São Paulo)</p> </li> </ul> <p>For a list of all the Amazon S3 supported
   * Regions and endpoints, see <a
   * href="https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region">Regions
   * and Endpoints</a> in the Amazon Web Services General Reference.</p>
   * </li> </ul> <p>For example, the following <code>x-amz-grant-read</code> header
   * grants the Amazon Web Services accounts identified by account IDs permissions to
   * read object data and its metadata:</p> <p> <code>x-amz-grant-read:
   * id="11112222333", id="444455556666" </code> </p>  <ul> <li> <p>This
   * functionality is not supported for directory buckets.</p> </li> <li> <p>This
   * functionality is not supported for Amazon S3 on Outposts.</p> </li> </ul>
   *
   */
  inline const Aws::String& GetGrantFullControl() const { return m_grantFullControl; }
  inline bool GrantFullControlHasBeenSet() const { return m_grantFullControlHasBeenSet; }
  template <typename GrantFullControlT = Aws::String>
  void SetGrantFullControl(GrantFullControlT&& value) {
    m_grantFullControlHasBeenSet = true;
    m_grantFullControl = std::forward<GrantFullControlT>(value);
  }
  template <typename GrantFullControlT = Aws::String>
  CreateMultipartUploadRequest& WithGrantFullControl(GrantFullControlT&& value) {
    SetGrantFullControl(std::forward<GrantFullControlT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specify access permissions explicitly to allow grantee to read the object
   * data and its metadata.</p> <p>By default, all objects are private. Only the
   * owner has full access control. When uploading an object, you can use this header
   * to explicitly grant access permissions to specific Amazon Web Services accounts
   * or groups. This header maps to specific permissions that Amazon S3 supports in
   * an ACL. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html">Access
   * Control List (ACL) Overview</a> in the <i>Amazon S3 User Guide</i>.</p> <p>You
   * specify each grantee as a type=value pair, where the type is one of the
   * following:</p> <ul> <li> <p> <code>id</code> – if the value specified is the
   * canonical user ID of an Amazon Web Services account</p> </li> <li> <p>
   * <code>uri</code> – if you are granting permissions to a predefined group</p>
   * </li> <li> <p> <code>emailAddress</code> – if the value specified is the email
   * address of an Amazon Web Services account</p>  <p>Using email addresses to
   * specify a grantee is only supported in the following Amazon Web Services
   * Regions: </p> <ul> <li> <p>US East (N. Virginia)</p> </li> <li> <p>US West (N.
   * California)</p> </li> <li> <p> US West (Oregon)</p> </li> <li> <p> Asia Pacific
   * (Singapore)</p> </li> <li> <p>Asia Pacific (Sydney)</p> </li> <li> <p>Asia
   * Pacific (Tokyo)</p> </li> <li> <p>Europe (Ireland)</p> </li> <li> <p>South
   * America (São Paulo)</p> </li> </ul> <p>For a list of all the Amazon S3 supported
   * Regions and endpoints, see <a
   * href="https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region">Regions
   * and Endpoints</a> in the Amazon Web Services General Reference.</p>
   * </li> </ul> <p>For example, the following <code>x-amz-grant-read</code> header
   * grants the Amazon Web Services accounts identified by account IDs permissions to
   * read object data and its metadata:</p> <p> <code>x-amz-grant-read:
   * id="11112222333", id="444455556666" </code> </p>  <ul> <li> <p>This
   * functionality is not supported for directory buckets.</p> </li> <li> <p>This
   * functionality is not supported for Amazon S3 on Outposts.</p> </li> </ul>
   *
   */
  inline const Aws::String& GetGrantRead() const { return m_grantRead; }
  inline bool GrantReadHasBeenSet() const { return m_grantReadHasBeenSet; }
  template <typename GrantReadT = Aws::String>
  void SetGrantRead(GrantReadT&& value) {
    m_grantReadHasBeenSet = true;
    m_grantRead = std::forward<GrantReadT>(value);
  }
  template <typename GrantReadT = Aws::String>
  CreateMultipartUploadRequest& WithGrantRead(GrantReadT&& value) {
    SetGrantRead(std::forward<GrantReadT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specify access permissions explicitly to allows grantee to read the object
   * ACL.</p> <p>By default, all objects are private. Only the owner has full access
   * control. When uploading an object, you can use this header to explicitly grant
   * access permissions to specific Amazon Web Services accounts or groups. This
   * header maps to specific permissions that Amazon S3 supports in an ACL. For more
   * information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html">Access
   * Control List (ACL) Overview</a> in the <i>Amazon S3 User Guide</i>.</p> <p>You
   * specify each grantee as a type=value pair, where the type is one of the
   * following:</p> <ul> <li> <p> <code>id</code> – if the value specified is the
   * canonical user ID of an Amazon Web Services account</p> </li> <li> <p>
   * <code>uri</code> – if you are granting permissions to a predefined group</p>
   * </li> <li> <p> <code>emailAddress</code> – if the value specified is the email
   * address of an Amazon Web Services account</p>  <p>Using email addresses to
   * specify a grantee is only supported in the following Amazon Web Services
   * Regions: </p> <ul> <li> <p>US East (N. Virginia)</p> </li> <li> <p>US West (N.
   * California)</p> </li> <li> <p> US West (Oregon)</p> </li> <li> <p> Asia Pacific
   * (Singapore)</p> </li> <li> <p>Asia Pacific (Sydney)</p> </li> <li> <p>Asia
   * Pacific (Tokyo)</p> </li> <li> <p>Europe (Ireland)</p> </li> <li> <p>South
   * America (São Paulo)</p> </li> </ul> <p>For a list of all the Amazon S3 supported
   * Regions and endpoints, see <a
   * href="https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region">Regions
   * and Endpoints</a> in the Amazon Web Services General Reference.</p>
   * </li> </ul> <p>For example, the following <code>x-amz-grant-read</code> header
   * grants the Amazon Web Services accounts identified by account IDs permissions to
   * read object data and its metadata:</p> <p> <code>x-amz-grant-read:
   * id="11112222333", id="444455556666" </code> </p>  <ul> <li> <p>This
   * functionality is not supported for directory buckets.</p> </li> <li> <p>This
   * functionality is not supported for Amazon S3 on Outposts.</p> </li> </ul>
   *
   */
  inline const Aws::String& GetGrantReadACP() const { return m_grantReadACP; }
  inline bool GrantReadACPHasBeenSet() const { return m_grantReadACPHasBeenSet; }
  template <typename GrantReadACPT = Aws::String>
  void SetGrantReadACP(GrantReadACPT&& value) {
    m_grantReadACPHasBeenSet = true;
    m_grantReadACP = std::forward<GrantReadACPT>(value);
  }
  template <typename GrantReadACPT = Aws::String>
  CreateMultipartUploadRequest& WithGrantReadACP(GrantReadACPT&& value) {
    SetGrantReadACP(std::forward<GrantReadACPT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specify access permissions explicitly to allows grantee to allow grantee to
   * write the ACL for the applicable object.</p> <p>By default, all objects are
   * private. Only the owner has full access control. When uploading an object, you
   * can use this header to explicitly grant access permissions to specific Amazon
   * Web Services accounts or groups. This header maps to specific permissions that
   * Amazon S3 supports in an ACL. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html">Access
   * Control List (ACL) Overview</a> in the <i>Amazon S3 User Guide</i>.</p> <p>You
   * specify each grantee as a type=value pair, where the type is one of the
   * following:</p> <ul> <li> <p> <code>id</code> – if the value specified is the
   * canonical user ID of an Amazon Web Services account</p> </li> <li> <p>
   * <code>uri</code> – if you are granting permissions to a predefined group</p>
   * </li> <li> <p> <code>emailAddress</code> – if the value specified is the email
   * address of an Amazon Web Services account</p>  <p>Using email addresses to
   * specify a grantee is only supported in the following Amazon Web Services
   * Regions: </p> <ul> <li> <p>US East (N. Virginia)</p> </li> <li> <p>US West (N.
   * California)</p> </li> <li> <p> US West (Oregon)</p> </li> <li> <p> Asia Pacific
   * (Singapore)</p> </li> <li> <p>Asia Pacific (Sydney)</p> </li> <li> <p>Asia
   * Pacific (Tokyo)</p> </li> <li> <p>Europe (Ireland)</p> </li> <li> <p>South
   * America (São Paulo)</p> </li> </ul> <p>For a list of all the Amazon S3 supported
   * Regions and endpoints, see <a
   * href="https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region">Regions
   * and Endpoints</a> in the Amazon Web Services General Reference.</p>
   * </li> </ul> <p>For example, the following <code>x-amz-grant-read</code> header
   * grants the Amazon Web Services accounts identified by account IDs permissions to
   * read object data and its metadata:</p> <p> <code>x-amz-grant-read:
   * id="11112222333", id="444455556666" </code> </p>  <ul> <li> <p>This
   * functionality is not supported for directory buckets.</p> </li> <li> <p>This
   * functionality is not supported for Amazon S3 on Outposts.</p> </li> </ul>
   *
   */
  inline const Aws::String& GetGrantWriteACP() const { return m_grantWriteACP; }
  inline bool GrantWriteACPHasBeenSet() const { return m_grantWriteACPHasBeenSet; }
  template <typename GrantWriteACPT = Aws::String>
  void SetGrantWriteACP(GrantWriteACPT&& value) {
    m_grantWriteACPHasBeenSet = true;
    m_grantWriteACP = std::forward<GrantWriteACPT>(value);
  }
  template <typename GrantWriteACPT = Aws::String>
  CreateMultipartUploadRequest& WithGrantWriteACP(GrantWriteACPT&& value) {
    SetGrantWriteACP(std::forward<GrantWriteACPT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Object key for which the multipart upload is to be initiated.</p>
   */
  inline const Aws::String& GetKey() const { return m_key; }
  inline bool KeyHasBeenSet() const { return m_keyHasBeenSet; }
  template <typename KeyT = Aws::String>
  void SetKey(KeyT&& value) {
    m_keyHasBeenSet = true;
    m_key = std::forward<KeyT>(value);
  }
  template <typename KeyT = Aws::String>
  CreateMultipartUploadRequest& WithKey(KeyT&& value) {
    SetKey(std::forward<KeyT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>A map of metadata to store with the object in S3.</p>
   */
  inline const Aws::Map<Aws::String, Aws::String>& GetMetadata() const { return m_metadata; }
  inline bool MetadataHasBeenSet() const { return m_metadataHasBeenSet; }
  template <typename MetadataT = Aws::Map<Aws::String, Aws::String>>
  void SetMetadata(MetadataT&& value) {
    m_metadataHasBeenSet = true;
    m_metadata = std::forward<MetadataT>(value);
  }
  template <typename MetadataT = Aws::Map<Aws::String, Aws::String>>
  CreateMultipartUploadRequest& WithMetadata(MetadataT&& value) {
    SetMetadata(std::forward<MetadataT>(value));
    return *this;
  }
  template <typename MetadataKeyT = Aws::String, typename MetadataValueT = Aws::String>
  CreateMultipartUploadRequest& AddMetadata(MetadataKeyT&& key, MetadataValueT&& value) {
    m_metadataHasBeenSet = true;
    m_metadata.emplace(std::forward<MetadataKeyT>(key), std::forward<MetadataValueT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The server-side encryption algorithm used when you store this object in
   * Amazon S3 or Amazon FSx.</p> <ul> <li> <p> <b>Directory buckets </b> - For
   * directory buckets, there are only two supported options for server-side
   * encryption: server-side encryption with Amazon S3 managed keys (SSE-S3)
   * (<code>AES256</code>) and server-side encryption with KMS keys (SSE-KMS)
   * (<code>aws:kms</code>). We recommend that the bucket's default encryption uses
   * the desired encryption configuration and you don't override the bucket default
   * encryption in your <code>CreateSession</code> requests or <code>PUT</code>
   * object requests. Then, new objects are automatically encrypted with the desired
   * encryption settings. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/s3-express-serv-side-encryption.html">Protecting
   * data with server-side encryption</a> in the <i>Amazon S3 User Guide</i>. For
   * more information about the encryption overriding behaviors in directory buckets,
   * see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/s3-express-specifying-kms-encryption.html">Specifying
   * server-side encryption with KMS for new object uploads</a>. </p> <p>In the Zonal
   * endpoint API calls (except <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/API/API_CopyObject.html">CopyObject</a>
   * and <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/API/API_UploadPartCopy.html">UploadPartCopy</a>)
   * using the REST API, the encryption request headers must match the encryption
   * settings that are specified in the <code>CreateSession</code> request. You can't
   * override the values of the encryption settings
   * (<code>x-amz-server-side-encryption</code>,
   * <code>x-amz-server-side-encryption-aws-kms-key-id</code>,
   * <code>x-amz-server-side-encryption-context</code>, and
   * <code>x-amz-server-side-encryption-bucket-key-enabled</code>) that are specified
   * in the <code>CreateSession</code> request. You don't need to explicitly specify
   * these encryption settings values in Zonal endpoint API calls, and Amazon S3 will
   * use the encryption settings values from the <code>CreateSession</code> request
   * to protect new objects in the directory bucket. </p>  <p>When you use the
   * CLI or the Amazon Web Services SDKs, for <code>CreateSession</code>, the session
   * token refreshes automatically to avoid service interruptions when a session
   * expires. The CLI or the Amazon Web Services SDKs use the bucket's default
   * encryption configuration for the <code>CreateSession</code> request. It's not
   * supported to override the encryption settings values in the
   * <code>CreateSession</code> request. So in the Zonal endpoint API calls (except
   * <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/API/API_CopyObject.html">CopyObject</a>
   * and <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/API/API_UploadPartCopy.html">UploadPartCopy</a>),
   * the encryption request headers must match the default encryption configuration
   * of the directory bucket. </p>  </li> <li> <p> <b>S3 access points for
   * Amazon FSx </b> - When accessing data stored in Amazon FSx file systems using S3
   * access points, the only valid server side encryption option is
   * <code>aws:fsx</code>. All Amazon FSx file systems have encryption configured by
   * default and are encrypted at rest. Data is automatically encrypted before being
   * written to the file system, and automatically decrypted as it is read. These
   * processes are handled transparently by Amazon FSx.</p> </li> </ul>
   */
  inline ServerSideEncryption GetServerSideEncryption() const { return m_serverSideEncryption; }
  inline bool ServerSideEncryptionHasBeenSet() const { return m_serverSideEncryptionHasBeenSet; }
  inline void SetServerSideEncryption(ServerSideEncryption value) {
    m_serverSideEncryptionHasBeenSet = true;
    m_serverSideEncryption = value;
  }
  inline CreateMultipartUploadRequest& WithServerSideEncryption(ServerSideEncryption value) {
    SetServerSideEncryption(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>By default, Amazon S3 uses the STANDARD Storage Class to store newly created
   * objects. The STANDARD storage class provides high durability and high
   * availability. Depending on performance needs, you can specify a different
   * Storage Class. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/dev/storage-class-intro.html">Storage
   * Classes</a> in the <i>Amazon S3 User Guide</i>.</p>  <ul> <li>
   * <p>Directory buckets only support <code>EXPRESS_ONEZONE</code> (the S3 Express
   * One Zone storage class) in Availability Zones and <code>ONEZONE_IA</code> (the
   * S3 One Zone-Infrequent Access storage class) in Dedicated Local Zones.</p> </li>
   * <li> <p>Amazon S3 on Outposts only uses the OUTPOSTS Storage Class.</p> </li>
   * </ul>
   */
  inline StorageClass GetStorageClass() const { return m_storageClass; }
  inline bool StorageClassHasBeenSet() const { return m_storageClassHasBeenSet; }
  inline void SetStorageClass(StorageClass value) {
    m_storageClassHasBeenSet = true;
    m_storageClass = value;
  }
  inline CreateMultipartUploadRequest& WithStorageClass(StorageClass value) {
    SetStorageClass(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>If the bucket is configured as a website, redirects requests for this object
   * to another object in the same bucket or to an external URL. Amazon S3 stores the
   * value of this header in the object metadata.</p>  <p>This functionality is
   * not supported for directory buckets.</p>
   */
  inline const Aws::String& GetWebsiteRedirectLocation() const { return m_websiteRedirectLocation; }
  inline bool WebsiteRedirectLocationHasBeenSet() const { return m_websiteRedirectLocationHasBeenSet; }
  template <typename WebsiteRedirectLocationT = Aws::String>
  void SetWebsiteRedirectLocation(WebsiteRedirectLocationT&& value) {
    m_websiteRedirectLocationHasBeenSet = true;
    m_websiteRedirectLocation = std::forward<WebsiteRedirectLocationT>(value);
  }
  template <typename WebsiteRedirectLocationT = Aws::String>
  CreateMultipartUploadRequest& WithWebsiteRedirectLocation(WebsiteRedirectLocationT&& value) {
    SetWebsiteRedirectLocation(std::forward<WebsiteRedirectLocationT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the algorithm to use when encrypting the object (for example,
   * AES256).</p>  <p>This functionality is not supported for directory
   * buckets.</p>
   */
  inline const Aws::String& GetSSECustomerAlgorithm() const { return m_sSECustomerAlgorithm; }
  inline bool SSECustomerAlgorithmHasBeenSet() const { return m_sSECustomerAlgorithmHasBeenSet; }
  template <typename SSECustomerAlgorithmT = Aws::String>
  void SetSSECustomerAlgorithm(SSECustomerAlgorithmT&& value) {
    m_sSECustomerAlgorithmHasBeenSet = true;
    m_sSECustomerAlgorithm = std::forward<SSECustomerAlgorithmT>(value);
  }
  template <typename SSECustomerAlgorithmT = Aws::String>
  CreateMultipartUploadRequest& WithSSECustomerAlgorithm(SSECustomerAlgorithmT&& value) {
    SetSSECustomerAlgorithm(std::forward<SSECustomerAlgorithmT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the customer-provided encryption key for Amazon S3 to use in
   * encrypting data. This value is used to store the object and then it is
   * discarded; Amazon S3 does not store the encryption key. The key must be
   * appropriate for use with the algorithm specified in the
   * <code>x-amz-server-side-encryption-customer-algorithm</code> header.</p>
   * <p>This functionality is not supported for directory buckets.</p>
   */
  inline const Aws::String& GetSSECustomerKey() const { return m_sSECustomerKey; }
  inline bool SSECustomerKeyHasBeenSet() const { return m_sSECustomerKeyHasBeenSet; }
  template <typename SSECustomerKeyT = Aws::String>
  void SetSSECustomerKey(SSECustomerKeyT&& value) {
    m_sSECustomerKeyHasBeenSet = true;
    m_sSECustomerKey = std::forward<SSECustomerKeyT>(value);
  }
  template <typename SSECustomerKeyT = Aws::String>
  CreateMultipartUploadRequest& WithSSECustomerKey(SSECustomerKeyT&& value) {
    SetSSECustomerKey(std::forward<SSECustomerKeyT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the 128-bit MD5 digest of the customer-provided encryption key
   * according to RFC 1321. Amazon S3 uses this header for a message integrity check
   * to ensure that the encryption key was transmitted without error.</p>
   * <p>This functionality is not supported for directory buckets.</p>
   */
  inline const Aws::String& GetSSECustomerKeyMD5() const { return m_sSECustomerKeyMD5; }
  inline bool SSECustomerKeyMD5HasBeenSet() const { return m_sSECustomerKeyMD5HasBeenSet; }
  template <typename SSECustomerKeyMD5T = Aws::String>
  void SetSSECustomerKeyMD5(SSECustomerKeyMD5T&& value) {
    m_sSECustomerKeyMD5HasBeenSet = true;
    m_sSECustomerKeyMD5 = std::forward<SSECustomerKeyMD5T>(value);
  }
  template <typename SSECustomerKeyMD5T = Aws::String>
  CreateMultipartUploadRequest& WithSSECustomerKeyMD5(SSECustomerKeyMD5T&& value) {
    SetSSECustomerKeyMD5(std::forward<SSECustomerKeyMD5T>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the KMS key ID (Key ID, Key ARN, or Key Alias) to use for object
   * encryption. If the KMS key doesn't exist in the same account that's issuing the
   * command, you must use the full Key ARN not the Key ID.</p> <p> <b>General
   * purpose buckets</b> - If you specify <code>x-amz-server-side-encryption</code>
   * with <code>aws:kms</code> or <code>aws:kms:dsse</code>, this header specifies
   * the ID (Key ID, Key ARN, or Key Alias) of the KMS key to use. If you specify
   * <code>x-amz-server-side-encryption:aws:kms</code> or
   * <code>x-amz-server-side-encryption:aws:kms:dsse</code>, but do not provide
   * <code>x-amz-server-side-encryption-aws-kms-key-id</code>, Amazon S3 uses the
   * Amazon Web Services managed key (<code>aws/s3</code>) to protect the data.</p>
   * <p> <b>Directory buckets</b> - To encrypt data using SSE-KMS, it's recommended
   * to specify the <code>x-amz-server-side-encryption</code> header to
   * <code>aws:kms</code>. Then, the
   * <code>x-amz-server-side-encryption-aws-kms-key-id</code> header implicitly uses
   * the bucket's default KMS customer managed key ID. If you want to explicitly set
   * the <code> x-amz-server-side-encryption-aws-kms-key-id</code> header, it must
   * match the bucket's default customer managed key (using key ID or ARN, not
   * alias). Your SSE-KMS configuration can only support 1 <a
   * href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#customer-cmk">customer
   * managed key</a> per directory bucket's lifetime. The <a
   * href="https://docs.aws.amazon.com/kms/latest/developerguide/concepts.html#aws-managed-cmk">Amazon
   * Web Services managed key</a> (<code>aws/s3</code>) isn't supported. Incorrect
   * key specification results in an HTTP <code>400 Bad Request</code> error. </p>
   */
  inline const Aws::String& GetSSEKMSKeyId() const { return m_sSEKMSKeyId; }
  inline bool SSEKMSKeyIdHasBeenSet() const { return m_sSEKMSKeyIdHasBeenSet; }
  template <typename SSEKMSKeyIdT = Aws::String>
  void SetSSEKMSKeyId(SSEKMSKeyIdT&& value) {
    m_sSEKMSKeyIdHasBeenSet = true;
    m_sSEKMSKeyId = std::forward<SSEKMSKeyIdT>(value);
  }
  template <typename SSEKMSKeyIdT = Aws::String>
  CreateMultipartUploadRequest& WithSSEKMSKeyId(SSEKMSKeyIdT&& value) {
    SetSSEKMSKeyId(std::forward<SSEKMSKeyIdT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the Amazon Web Services KMS Encryption Context to use for object
   * encryption. The value of this header is a Base64 encoded string of a UTF-8
   * encoded JSON, which contains the encryption context as key-value pairs.</p> <p>
   * <b>Directory buckets</b> - You can optionally provide an explicit encryption
   * context value. The value must match the default encryption context - the bucket
   * Amazon Resource Name (ARN). An additional encryption context value is not
   * supported. </p>
   */
  inline const Aws::String& GetSSEKMSEncryptionContext() const { return m_sSEKMSEncryptionContext; }
  inline bool SSEKMSEncryptionContextHasBeenSet() const { return m_sSEKMSEncryptionContextHasBeenSet; }
  template <typename SSEKMSEncryptionContextT = Aws::String>
  void SetSSEKMSEncryptionContext(SSEKMSEncryptionContextT&& value) {
    m_sSEKMSEncryptionContextHasBeenSet = true;
    m_sSEKMSEncryptionContext = std::forward<SSEKMSEncryptionContextT>(value);
  }
  template <typename SSEKMSEncryptionContextT = Aws::String>
  CreateMultipartUploadRequest& WithSSEKMSEncryptionContext(SSEKMSEncryptionContextT&& value) {
    SetSSEKMSEncryptionContext(std::forward<SSEKMSEncryptionContextT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies whether Amazon S3 should use an S3 Bucket Key for object encryption
   * with server-side encryption using Key Management Service (KMS) keys
   * (SSE-KMS).</p> <p> <b>General purpose buckets</b> - Setting this header to
   * <code>true</code> causes Amazon S3 to use an S3 Bucket Key for object encryption
   * with SSE-KMS. Also, specifying this header with a PUT action doesn't affect
   * bucket-level settings for S3 Bucket Key.</p> <p> <b>Directory buckets</b> - S3
   * Bucket Keys are always enabled for <code>GET</code> and <code>PUT</code>
   * operations in a directory bucket and can’t be disabled. S3 Bucket Keys aren't
   * supported, when you copy SSE-KMS encrypted objects from general purpose buckets
   * to directory buckets, from directory buckets to general purpose buckets, or
   * between directory buckets, through <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/API/API_CopyObject.html">CopyObject</a>,
   * <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/API/API_UploadPartCopy.html">UploadPartCopy</a>,
   * <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/directory-buckets-objects-Batch-Ops">the
   * Copy operation in Batch Operations</a>, or <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/create-import-job">the
   * import jobs</a>. In this case, Amazon S3 makes a call to KMS every time a copy
   * request is made for a KMS-encrypted object.</p>
   */
  inline bool GetBucketKeyEnabled() const { return m_bucketKeyEnabled; }
  inline bool BucketKeyEnabledHasBeenSet() const { return m_bucketKeyEnabledHasBeenSet; }
  inline void SetBucketKeyEnabled(bool value) {
    m_bucketKeyEnabledHasBeenSet = true;
    m_bucketKeyEnabled = value;
  }
  inline CreateMultipartUploadRequest& WithBucketKeyEnabled(bool value) {
    SetBucketKeyEnabled(value);
    return *this;
  }
  ///@}

  ///@{

  inline RequestPayer GetRequestPayer() const { return m_requestPayer; }
  inline bool RequestPayerHasBeenSet() const { return m_requestPayerHasBeenSet; }
  inline void SetRequestPayer(RequestPayer value) {
    m_requestPayerHasBeenSet = true;
    m_requestPayer = value;
  }
  inline CreateMultipartUploadRequest& WithRequestPayer(RequestPayer value) {
    SetRequestPayer(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The tag-set for the object. The tag-set must be encoded as URL Query
   * parameters.</p>  <p>This functionality is not supported for directory
   * buckets.</p>
   */
  inline const Aws::String& GetTagging() const { return m_tagging; }
  inline bool TaggingHasBeenSet() const { return m_taggingHasBeenSet; }
  template <typename TaggingT = Aws::String>
  void SetTagging(TaggingT&& value) {
    m_taggingHasBeenSet = true;
    m_tagging = std::forward<TaggingT>(value);
  }
  template <typename TaggingT = Aws::String>
  CreateMultipartUploadRequest& WithTagging(TaggingT&& value) {
    SetTagging(std::forward<TaggingT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the Object Lock mode that you want to apply to the uploaded
   * object.</p>  <p>This functionality is not supported for directory
   * buckets.</p>
   */
  inline ObjectLockMode GetObjectLockMode() const { return m_objectLockMode; }
  inline bool ObjectLockModeHasBeenSet() const { return m_objectLockModeHasBeenSet; }
  inline void SetObjectLockMode(ObjectLockMode value) {
    m_objectLockModeHasBeenSet = true;
    m_objectLockMode = value;
  }
  inline CreateMultipartUploadRequest& WithObjectLockMode(ObjectLockMode value) {
    SetObjectLockMode(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies the date and time when you want the Object Lock to expire.</p>
   *  <p>This functionality is not supported for directory buckets.</p>
   */
  inline const Aws::Utils::DateTime& GetObjectLockRetainUntilDate() const { return m_objectLockRetainUntilDate; }
  inline bool ObjectLockRetainUntilDateHasBeenSet() const { return m_objectLockRetainUntilDateHasBeenSet; }
  template <typename ObjectLockRetainUntilDateT = Aws::Utils::DateTime>
  void SetObjectLockRetainUntilDate(ObjectLockRetainUntilDateT&& value) {
    m_objectLockRetainUntilDateHasBeenSet = true;
    m_objectLockRetainUntilDate = std::forward<ObjectLockRetainUntilDateT>(value);
  }
  template <typename ObjectLockRetainUntilDateT = Aws::Utils::DateTime>
  CreateMultipartUploadRequest& WithObjectLockRetainUntilDate(ObjectLockRetainUntilDateT&& value) {
    SetObjectLockRetainUntilDate(std::forward<ObjectLockRetainUntilDateT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Specifies whether you want to apply a legal hold to the uploaded object.</p>
   *  <p>This functionality is not supported for directory buckets.</p>
   */
  inline ObjectLockLegalHoldStatus GetObjectLockLegalHoldStatus() const { return m_objectLockLegalHoldStatus; }
  inline bool ObjectLockLegalHoldStatusHasBeenSet() const { return m_objectLockLegalHoldStatusHasBeenSet; }
  inline void SetObjectLockLegalHoldStatus(ObjectLockLegalHoldStatus value) {
    m_objectLockLegalHoldStatusHasBeenSet = true;
    m_objectLockLegalHoldStatus = value;
  }
  inline CreateMultipartUploadRequest& WithObjectLockLegalHoldStatus(ObjectLockLegalHoldStatus value) {
    SetObjectLockLegalHoldStatus(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>The account ID of the expected bucket owner. If the account ID that you
   * provide does not match the actual owner of the bucket, the request fails with
   * the HTTP status code <code>403 Forbidden</code> (access denied).</p>
   */
  inline const Aws::String& GetExpectedBucketOwner() const { return m_expectedBucketOwner; }
  inline bool ExpectedBucketOwnerHasBeenSet() const { return m_expectedBucketOwnerHasBeenSet; }
  template <typename ExpectedBucketOwnerT = Aws::String>
  void SetExpectedBucketOwner(ExpectedBucketOwnerT&& value) {
    m_expectedBucketOwnerHasBeenSet = true;
    m_expectedBucketOwner = std::forward<ExpectedBucketOwnerT>(value);
  }
  template <typename ExpectedBucketOwnerT = Aws::String>
  CreateMultipartUploadRequest& WithExpectedBucketOwner(ExpectedBucketOwnerT&& value) {
    SetExpectedBucketOwner(std::forward<ExpectedBucketOwnerT>(value));
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Indicates the algorithm that you want Amazon S3 to use to create the checksum
   * for the object. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html">Checking
   * object integrity</a> in the <i>Amazon S3 User Guide</i>.</p>
   */
  inline ChecksumAlgorithm GetChecksumAlgorithm() const { return m_checksumAlgorithm; }
  inline bool ChecksumAlgorithmHasBeenSet() const { return m_checksumAlgorithmHasBeenSet; }
  inline void SetChecksumAlgorithm(ChecksumAlgorithm value) {
    m_checksumAlgorithmHasBeenSet = true;
    m_checksumAlgorithm = value;
  }
  inline CreateMultipartUploadRequest& WithChecksumAlgorithm(ChecksumAlgorithm value) {
    SetChecksumAlgorithm(value);
    return *this;
  }
  ///@}

  ///@{
  /**
   * <p>Indicates the checksum type that you want Amazon S3 to use to calculate the
   * object’s checksum value. For more information, see <a
   * href="https://docs.aws.amazon.com/AmazonS3/latest/userguide/checking-object-integrity.html">Checking
   * object integrity in the Amazon S3 User Guide</a>.</p>
   */
  inline ChecksumType GetChecksumType() const { return m_checksumType; }
  inline bool ChecksumTypeHasBeenSet() const { return m_checksumTypeHasBeenSet; }
  inline void SetChecksumType(ChecksumType value) {
    m_checksumTypeHasBeenSet = true;
    m_checksumType = value;
  }
  inline CreateMultipartUploadRequest& WithChecksumType(ChecksumType value) {
    SetChecksumType(value);
    return *this;
  }
  ///@}

  ///@{

  inline const Aws::Map<Aws::String, Aws::String>& GetCustomizedAccessLogTag() const { return m_customizedAccessLogTag; }
  inline bool CustomizedAccessLogTagHasBeenSet() const { return m_customizedAccessLogTagHasBeenSet; }
  template <typename CustomizedAccessLogTagT = Aws::Map<Aws::String, Aws::String>>
  void SetCustomizedAccessLogTag(CustomizedAccessLogTagT&& value) {
    m_customizedAccessLogTagHasBeenSet = true;
    m_customizedAccessLogTag = std::forward<CustomizedAccessLogTagT>(value);
  }
  template <typename CustomizedAccessLogTagT = Aws::Map<Aws::String, Aws::String>>
  CreateMultipartUploadRequest& WithCustomizedAccessLogTag(CustomizedAccessLogTagT&& value) {
    SetCustomizedAccessLogTag(std::forward<CustomizedAccessLogTagT>(value));
    return *this;
  }
  template <typename CustomizedAccessLogTagKeyT = Aws::String, typename CustomizedAccessLogTagValueT = Aws::String>
  CreateMultipartUploadRequest& AddCustomizedAccessLogTag(CustomizedAccessLogTagKeyT&& key, CustomizedAccessLogTagValueT&& value) {
    m_customizedAccessLogTagHasBeenSet = true;
    m_customizedAccessLogTag.emplace(std::forward<CustomizedAccessLogTagKeyT>(key), std::forward<CustomizedAccessLogTagValueT>(value));
    return *this;
  }
  ///@}
 private:
  ObjectCannedACL m_aCL{ObjectCannedACL::NOT_SET};

  Aws::String m_bucket;

  Aws::String m_cacheControl;

  Aws::String m_contentDisposition;

  Aws::String m_contentEncoding;

  Aws::String m_contentLanguage;

  Aws::String m_contentType;

  Aws::Utils::DateTime m_expires{};

  Aws::String m_grantFullControl;

  Aws::String m_grantRead;

  Aws::String m_grantReadACP;

  Aws::String m_grantWriteACP;

  Aws::String m_key;

  Aws::Map<Aws::String, Aws::String> m_metadata;

  ServerSideEncryption m_serverSideEncryption{ServerSideEncryption::NOT_SET};

  StorageClass m_storageClass{StorageClass::NOT_SET};

  Aws::String m_websiteRedirectLocation;

  Aws::String m_sSECustomerAlgorithm;

  Aws::String m_sSECustomerKey;

  Aws::String m_sSECustomerKeyMD5;

  Aws::String m_sSEKMSKeyId;

  Aws::String m_sSEKMSEncryptionContext;

  bool m_bucketKeyEnabled{false};

  RequestPayer m_requestPayer{RequestPayer::NOT_SET};

  Aws::String m_tagging;

  ObjectLockMode m_objectLockMode{ObjectLockMode::NOT_SET};

  Aws::Utils::DateTime m_objectLockRetainUntilDate{};

  ObjectLockLegalHoldStatus m_objectLockLegalHoldStatus{ObjectLockLegalHoldStatus::NOT_SET};

  Aws::String m_expectedBucketOwner;

  ChecksumAlgorithm m_checksumAlgorithm{ChecksumAlgorithm::NOT_SET};

  ChecksumType m_checksumType{ChecksumType::NOT_SET};

  Aws::Map<Aws::String, Aws::String> m_customizedAccessLogTag;
  bool m_aCLHasBeenSet = false;
  bool m_bucketHasBeenSet = false;
  bool m_cacheControlHasBeenSet = false;
  bool m_contentDispositionHasBeenSet = false;
  bool m_contentEncodingHasBeenSet = false;
  bool m_contentLanguageHasBeenSet = false;
  bool m_contentTypeHasBeenSet = false;
  bool m_expiresHasBeenSet = false;
  bool m_grantFullControlHasBeenSet = false;
  bool m_grantReadHasBeenSet = false;
  bool m_grantReadACPHasBeenSet = false;
  bool m_grantWriteACPHasBeenSet = false;
  bool m_keyHasBeenSet = false;
  bool m_metadataHasBeenSet = false;
  bool m_serverSideEncryptionHasBeenSet = false;
  bool m_storageClassHasBeenSet = false;
  bool m_websiteRedirectLocationHasBeenSet = false;
  bool m_sSECustomerAlgorithmHasBeenSet = false;
  bool m_sSECustomerKeyHasBeenSet = false;
  bool m_sSECustomerKeyMD5HasBeenSet = false;
  bool m_sSEKMSKeyIdHasBeenSet = false;
  bool m_sSEKMSEncryptionContextHasBeenSet = false;
  bool m_bucketKeyEnabledHasBeenSet = false;
  bool m_requestPayerHasBeenSet = false;
  bool m_taggingHasBeenSet = false;
  bool m_objectLockModeHasBeenSet = false;
  bool m_objectLockRetainUntilDateHasBeenSet = false;
  bool m_objectLockLegalHoldStatusHasBeenSet = false;
  bool m_expectedBucketOwnerHasBeenSet = false;
  bool m_checksumAlgorithmHasBeenSet = false;
  bool m_checksumTypeHasBeenSet = false;
  bool m_customizedAccessLogTagHasBeenSet = false;
};

}  // namespace Model
}  // namespace S3
}  // namespace Aws
